#ifndef EIGEN_PPI_CPP_
#define EIGEN_PPI_CPP_
#include "eigen_ppi.h"

void expand_g(){
	map<int,bool>adj;
	map<int,double>new_adj;
	map<int,bool>exclude_adj;
	int tot_cntr=0;
	double NEW_FLAG=-1000000;
	double PAIRING_TH=1-1e-3;
	
	for(int i=0;i<dim;i++){
		if(nodeList[i]->degC==0)continue;
		adj.clear();new_adj.clear();exclude_adj.clear();
		for(int j=0;j<nodeList[i]->deg;j++){
			if(!isHom[i][j])adj[matrixind[i][j]]=true;
		}
		for(int j=0;j<nodeList[i]->deg;j++){
			if(isHom[i][j])continue;
			int v=matrixind[i][j];
			for(int k=0;k<nodeList[v]->deg;k++){
				if(matrix[v][k]<NEW_FLAG || isHom[v][k])continue;
				int u=matrixind[v][k];	
				if(u==i || adj.find(u)!=adj.end() || 
					new_adj.find(u)!=new_adj.end() || 
					exclude_adj.find(u)!=exclude_adj.end())continue;
				double tmpv=(1-matrix[i][j])*(1-matrix[v][k]);//assume matrix holds 1-p
				if(tmpv>PAIRING_TH)new_adj[u]=1-0.5*tmpv;
				else exclude_adj[u]=true;
			}
		}
		if(new_adj.size()>0){
			tot_cntr+=new_adj.size();
			int curr_deg=new_adj.size()+nodeList[i]->deg;
			double *newVal=new double[curr_deg];
			double *newTVal=new double[curr_deg];
			int *new_Mind=new int[curr_deg];
			bool *newIsHom=new bool[curr_deg];
			int j=0;
			for(;j<nodeList[i]->deg;j++){
				newVal[j]=matrix[i][j];
				new_Mind[j]=matrixind[i][j];
				newIsHom[j]=isHom[i][j];
			}
			for(map<int,double>::iterator ii=new_adj.begin();
				ii!=new_adj.end();ii++){
				newVal[j]=NEW_FLAG-ii->second;
				new_Mind[j]=ii->first;
				newIsHom[j]=false;
				j++;
			}
			nodeList[i]->deg+=new_adj.size();
			nodeList[i]->degC+=new_adj.size();
			delete[](matrix[i]);delete[](matrixind[i]);
			delete[](isHom[i]);delete[](Tmatrix[i]);
			matrix[i]=newVal;matrixind[i]=new_Mind;
			isHom[i]=newIsHom;Tmatrix[i]=newTVal;
		}
	}
	isOriginal=new bool*[dim];
	for(int i=0;i<dim;i++){
		isOriginal[i]=new bool[nodeList[i]->deg];
		for(int j=0;j<nodeList[i]->deg;j++){
			if(isHom[i][j])isOriginal[i][j]=true;
			else
			if(matrix[i][j]<NEW_FLAG){
				matrix[i][j]=NEW_FLAG-matrix[i][j];
				isOriginal[i][j]=false;
				if(i<matrixind[i][j])tot_ppi+=matrix[i][j];
			}else isOriginal[i][j]=true;
		}
	}
	printf("Added %d/2 edges\n",tot_cntr);
}

void normalize_eig(double ** Tmat){
	printf("Normalize, tot_ppi=%g\n",tot_ppi);
	double sum_hom=0,sum_ppi=0;
	for(int j=0;j<dim;j++){
		sum_hom=0;
		for(int i=0;i<nodeList[j]->deg;i++)  // calc exp, add up 
		{
			if(isHom[j][i]){
				Tmat[j][i]=matrix[j][i];
				sum_hom+=Tmat[j][i];
			}
			else {
				//Tmat[j][i]=(1-matrix[j][i])/tot_ppi;
				Tmat[j][i]=matrix[j][i];
				sum_ppi+=Tmat[j][i];
			} 
		}
		if(sum_hom>1e-20) 
			for(int i=0;i<nodeList[j]->deg;i++){ 
		  		if(isHom[j][i]){
		  			Tmat[j][i]/=sum_hom*2;
		  			if(Tmat[j][i]<TINY)Tmat[j][i]=TINY;
		  		}
			}
		if(sum_ppi>1e-20) 
			for(int i=0;i<nodeList[j]->deg;i++){ 
		  		if(!isHom[j][i]){
		  			Tmat[j][i]/=sum_ppi*2;
		  			if(Tmat[j][i]<TINY)Tmat[j][i]=TINY;
		  		}
			}				
	}	
}


void run_eig_ppi(score_map *eprob){
	int num,v,x,y;double tmp;
	score_map eprob_prev;
	score_map initProb;
	double sumdiff=0;
	for(int i=0;i<dim;i++){
		for(int j=0;j<nodeList[i]->deg;j++){
			if(isHom[i][j] || i>=matrixind[i][j])continue;
			(*eprob)[i*dim+matrixind[i][j]]=Tmatrix[i][j];
			initProb[i*dim+matrixind[i][j]]=(1-matrix[j][i])/tot_ppi;//Tmatrix[i][j];
		}
	}
	double *tmparr1=new double[dim*dim];
	double *tmparr2=new double[dim*dim];
	int tmpcntr=0;
	printf("Initial weight calculated for %d PPI candidates\n",initProb.size());
	int loops=0;for(;loops<MAX_LOOP;loops++)  
   	{   
   		//printf("========Loop %d=======\n",loops);
   		printf(".");fflush(stdout);sumdiff=0;
   		for(score_map::iterator ii=(*eprob).begin();
   			ii!=(*eprob).end();ii++){
   				if(loops!=0)sumdiff+=pow(eprob_prev[ii->first]-ii->second,2);
   				eprob_prev[ii->first]=ii->second;
   		}if(loops!=0 && sqrt(sumdiff)<1e-50)break;
   		for(int u=0;u<dim;u++){
   			for(int j=0;j<nodeList[u]->deg;j++){
				if(matrixind[u][j]<u || isHom[u][j])continue; //u<v
				v=matrixind[u][j];tmp=0;
				for(int x1=0;x1<nodeList[u]->deg;x1++){
					//if(!isHom[u][x1])continue;
					for(int y1=0;y1<nodeList[v]->deg;y1++){
						if(isHom[v][y1]==isHom[u][x1]){
							x=matrixind[u][x1];
							y=matrixind[v][y1];
							if(x<y)num=x*dim+y;else num=y*dim+x;
							if((*eprob).find(num)!=(*eprob).end()){
								tmp+=Tmatrix[u][x1]*Tmatrix[v][y1]*eprob_prev[num];
							} 	
						}
					}
				}
				(*eprob)[u*dim+v]=alpha*tmp + (1-alpha)*initProb[u*dim+v];
				if(loops==19 && tmpcntr<dim*dim){
					tmparr1[tmpcntr]=tmp;
					tmparr2[tmpcntr++]=initProb[u*dim+v];
				}
				/*printf("Pr[%s,%s]=alpha*%g + (1-alpha)*%g = %g\n",
					nodeList[u]->name,nodeList[v]->name,tmp,
					initProb[u*dim+v],(*eprob)[u*dim+v]);*/
			}
   		}
   	}
   	printf("\nStopped after %d loops. Norm-2 = %g\n",loops,sqrt(sumdiff));
   	fflush(stdout);
   	
   	FILE *f1=fopen("local","w");
   	for(int i=0;i<tmpcntr;i++)fprintf(f1,"%g\n",tmparr2[i]);
   	fclose(f1);
   	f1=fopen("global","w");
   	for(int i=0;i<tmpcntr;i++)fprintf(f1,"%g\n",tmparr1[i]);
   	fclose(f1);
   	delete[]tmparr1;delete[]tmparr2;
}


void write_eig_ppi(score_map eprob){
	  char fname[NAME_LEN];int q,u,v,gstype;
	  sprintf(fname,"%s/dtls/vals.ppi.0",postfix);
	  FILE *fpValC=fopen(fname,"w");
	  sprintf(fname,"%s/dtls/label.ppi.0",postfix);
	  FILE *fpLabC=fopen(fname,"w");
	  sprintf(fname,"%s/dtls/ranks.ppi.0",postfix);
	  FILE *fpRC=fopen(fname,"w");
	  
	  for(map<int,bool>::iterator kk=Qmap.begin();kk!=Qmap.end();kk++) 
	  {
		  q=kk->first;u=q;
		  fprintf(fpValC,"%s",nodeList[q]->name);
		  fprintf(fpLabC,"%s",nodeList[q]->name);
		  fprintf(fpRC,">%s(",nodeList[q]->name);
		  for(nlist::iterator ii=nodeList[q]->GSC.begin();ii!=nodeList[q]->GSC.end();ii++){
  				fprintf(fpRC,"%s-%d, ",ii->first->name,ii->second);;
	      }fprintf(fpRC,")\n");
	      double tmparr[nodeList[q]->deg];int len=0;
	      int tmparr_ind[nodeList[q]->deg];
	      for(int j=0;j<nodeList[q]->deg;j++){
	      	//if(isHom[q][j] || !isOriginal[q][j])continue;
	      	if(isHom[q][j])continue;
	      	v=matrixind[q][j];
	      	tmparr_ind[len]=j;
			u=q;if(v<u){int tmp=u;u=v;v=tmp;}
			tmparr[len++]=-eprob[u*dim+v];
	      }
	      mergeSort(tmparr,tmparr_ind,len,true);
	      
 	  	  for(int jj=0;jj<len;jj++){
 	  	  	int j=tmparr_ind[jj];
			if(isHom[q][j])continue;
			v=matrixind[q][j];
			if(nodeList[q]->GSC.find(nodeList[v])!=nodeList[q]->GSC.end())
				gstype=nodeList[q]->GSC[nodeList[v]];
			else gstype=2;
			u=q;if(v<u){int tmp=u;u=v;v=tmp;}
			fprintf(fpValC," %g",eprob[u*dim+v]);
		  	fprintf(fpLabC," %d",gstype);
		  	fprintf(fpRC,"%s %g [%d]\n",nodeList[matrixind[q][j]]->name,
		  		eprob[u*dim+v],gstype);
		  }
		  fprintf(fpValC,"\n");
		  fprintf(fpLabC,"\n");
	}
	fclose(fpLabC);fclose(fpValC);fclose(fpRC);
}


void eig_ppi(char *homDir){
  char fH[MAX_SPEC][NAME_LEN],fC[MAX_SPEC][NAME_LEN],Qfile[NAME_LEN],
  	GSCfile[NAME_LEN],GSHfile[NAME_LEN];
  int sH[MAX_SPEC][2],sC[MAX_SPEC][2];
  int nH,nC,ver=0,win_c=-1;
  char fname[NAME_LEN];
  sprintf(fname,"%s/config_large_scale",homDir);
  load_config(fname,fH,&nH,sH,fC,&nC,sC,Qfile,GSCfile,GSHfile);
  load_matrix(fC,sC,nC,fH,sH,nH,true);
  printf("Config: %d PPI files and %d homology files\n",nC,nH);fflush(stdout);
  alpha=.95;
  loadGS(GSCfile,false);
  //printf("expand_g\n");fflush(stdout);
  //expand_g();
  printf("normalize_eig\n");fflush(stdout);
  normalize_eig(Tmatrix);
  printf("run_eig_ppi\n");fflush(stdout);
  score_map eprob;   
  run_eig_ppi(&eprob);
  printf("write_eig_ppi\n");fflush(stdout);
  write_eig_ppi(eprob);
  clean();	
}


#endif


