#include "rankp.h"
#include "util/util.h"
#include "util/io_util.h"
#include "analyze_out.h"
#include "eigen_ppi.h"
/*parameters:
  	alpha - controls amount of "clustering"
	sigmaC - width of PPI confidence transformation 
	sigmaH - width of PSI-BLAST E-value transformation
	rho - balance between homology and interaction edges
*/
double sigmaH,sigmaC,rho,alpha;//0.95;//0;
bool ONLY_INTERCATING_PROT=false;//true;//
int MAX_LOOP=20;
int OPT1,OPT2;
int dataset;
int IDPROT=0;
double tot_ppi=0;
int TASK,REL_TYPE,Qalg_FLAG=-1;
int dim;    // dimension of database
double **matrix;    // edge weights
double **Tmatrix;   //Transformed edge weights
double **Tmatrix_dppi; //Transformed edge weights for delta PPI method
int **matrixind;   // incoming edge target IDs
int **matrixind_out; // outgoing edge target IDs
bool **isHom; //edge type indicator
bool **isOriginal;//length-1 paths
double **PairWiseMat,*G0,**PairWiseMat_help,**W_mul_G0,**aggregateMat,*PRank_vec;
bool computed_PairWise=false,allocated_PairWise=false;

map <int,bool>Qmap;//Queries
char postfix[256];
double MAXc=TINY,MAXh=TINY;
Xref nodeXref; 
node** nodeList;

bool ltstr::operator ()(const char* s1, const char* s2) const
  {
    return strcmp(s1, s2) < 0;
  }


bool ltvoid::operator()(void* s1, void* s2) const
  {
      return(s1<s2);
  }

node::node(short spec2,int deg1,char *name1){
		deg=deg1;
		id=IDPROT++;
		spec=spec2;
		name=name1;
		roc_score[0]=0;roc_score[1]=0;degC=0;
		degOut=0;
};



double tf1(double x,double sigma){
	return 	exp(-x/sigma);
}
double tf2(double x,double sigma){
	//if(fpclassify(xx)==FP_NAN)
	return -log(x) +log(sigma); //<=> -log(x/(MAX*sigma))
}
double tf2_comp(double x,double sigma){
	return sigma*exp(-x);
}



void normalize(double ** Tmat,bool PPIflag,int q,int prev_q) 
{ 
	  int i,j; double sum_hom,sum_ppi;
	  j=1;double fact_hom,fact_ppi;
	  int list[dim];int len;
	  if(prev_q==-1){for(j=0;j<dim;j++)list[j]=j;len=dim;}
	  else{
	  	len=0;
	  	for(i=0;i<nodeList[prev_q]->degOut;i++)list[len++]=matrixind_out[prev_q][i];
	  	list[len++]=prev_q;
	  	for(i=0;i<nodeList[q]->degOut;i++)list[len++]=matrixind_out[q][i];
	  }
	  for(j=0;j<len;j++){  // do each row in turn ...
	  	normalize_node(Tmat,PPIflag,list[j],q);
	  }
} 


void normalize_node(double **Tmat,bool PPIflag,int j,int q){
		double sum_hom=0,sum_ppi=0;int i;double fact_hom,fact_ppi;
	    if(OPT2==6){
	    	for(i=0;i<nodeList[j]->deg;i++)Tmat[j][i]=matrix[j][i];
	    	return;
	    }
	    for(i=0;i<nodeList[j]->deg;i++)  // calc exp, add up 
		{
				if(isHom[j][i]){
					Tmat[j][i]=tf2(matrix[j][i],sigmaH*MAXh);
					if(matrixind[j][i]!=q)sum_hom+=Tmat[j][i];
				}
				else {
					Tmat[j][i]=tf2(matrix[j][i],sigmaC*MAXc);
					if(matrixind[j][i]!=q)sum_ppi+=Tmat[j][i];
				} 
		}
		//ppi abundance should not effet homology weight
		fact_ppi=(1/(1+rho));
		if(PPIflag)fact_hom=(rho/(1+rho));else fact_hom=1;
		if(sum_hom>1e-20) 
			for(i=0;i<nodeList[j]->deg;i++){ 
		  		if(isHom[j][i]){
		  			if(matrixind[j][i]!=q){
		  				Tmat[j][i]=fact_hom*Tmat[j][i]/sum_hom;
		  				if(Tmat[j][i]<TINY)Tmat[j][i]=TINY;
		  			}
		  		}
			}
		if(sum_ppi>1e-20)  
			for(i=0;i<nodeList[j]->deg;i++){ 
		  		if(!isHom[j][i]){
		  			if(PPIflag){
		  				if(matrixind[j][i]!=q){
		  					Tmat[j][i]=fact_ppi*Tmat[j][i]/sum_ppi;
		  					if(Tmat[j][i]<TINY)Tmat[j][i]=TINY;
		  				}
					}
		  			else Tmat[j][i]=0;
		  		}
			}	
}

void normalizeZSR(double ** Tmat){
	double sum_hom[dim],sum_ppi[dim];
	for(int j=0;j<dim;j++){
		sum_hom[j]=1e-20;sum_ppi[j]=1e-20;
		for(int i=0;i<nodeList[j]->deg;i++)  // calc exp, add up 
		{
			if(isHom[j][i]){
				Tmat[j][i]=tf2(matrix[j][i],sigmaH*MAXh);
				sum_hom[j]+=Tmat[j][i];
			}
			else {
				Tmat[j][i]=tf2(matrix[j][i],sigmaC*MAXc);//1-Tmat[j][i];//
				sum_ppi[j]+=Tmat[j][i];
			} 
		}
	}
	double fact_hom,fact_ppi=(1/(1+rho));
	fact_hom=(rho/(1+rho));
	for(int j=0;j<dim;j++){
		for(int i=0;i<nodeList[j]->deg;i++){ 
			if(isHom[j][i]){
				Tmat[j][i]=fact_hom*Tmat[j][i]/sqrt(sum_hom[j]*sum_hom[i]);
		  		if(Tmat[j][i]<TINY)Tmat[j][i]=TINY;
			}
		}
	}
	for(int j=0;j<dim;j++){
		for(int i=0;i<nodeList[j]->deg;i++){ 
			if(!isHom[j][i]){
		  		Tmat[j][i]=fact_ppi*Tmat[j][i]/sqrt(sum_ppi[j]*sum_ppi[i]);
		  		if(Tmat[j][i]<TINY)Tmat[j][i]=TINY;
			}
		}
	}
}
void diffuse(double *y,int k,bool PPIflag,double ** Tmat){
	//PPIflag use PPI connectivity
	//printf("Diffuse from node %d (%s)\n",k,nodeList[k]->name);fflush(stdout);
	double *yold=new double[dim];
	double *yorig=new double[dim];
	double *row_eval;bool *isHom_r;
	int *row_ind,i,j;
	double diff,STOP_CRITERIA=1e-10;
	//int tmpid[]={nodeXref["GI|1101029"]->id,nodeXref["GI|1583223"]->id,nodeXref["GI|19344004"]->id};
	for(i=0;i<dim;i++)   // setup labels 
		{y[i]=0.0; yorig[i]=0;}     
	row_ind=matrixind_out[k]; 
	//row_eval=matrix[k];
  	int spec=nodeList[k]->spec;
  	for(i=0;i<nodeList[k]->degOut;i++){
  	//printf("testing adj no.%d (%d)\n",i,nodeList[row_ind[i]]->id);fflush(stdout);   
    	if(PPIflag || nodeList[row_ind[i]]->spec!=spec){
    		bool found=false; 
    		for(int j=0;!found&&j<nodeList[row_ind[i]]->deg;j++){
    			if(matrixind[row_ind[i]][j]==k){
    				yorig[row_ind[i]]=Tmat[row_ind[i]][j];
    				found=true;
    			}
    		}
    		if(!found){
    			printf("Could not find query %s in list of %s (i=%d)\n",nodeList[k]->name,nodeList[row_ind[i]]->name,i);
    			my_exit("diffuse:integrity\n");
    		}
    	}
  	}
  	//row_ind=matrixind[k];
  	/*printf("deg: %d\n",nodeList[k]->degOut);
    	for(i=0;i<nodeList[k]->degOut;i++){
    		printf("%d: %s-%s,%g(%g) %g\n",
	    		i+1,nodeList[k]->name,nodeList[row_ind[i]]->name,Tmat[k][i],yorig[row_ind[i]],matrix[k][i]);}
  	*/
  	for(int loops=0;loops<MAX_LOOP;loops++)  // iterations of propagation 
   	{   
   		//printf("===== loop %d ========\n",loops);
		diff=0;for(i=0;i<dim;i++) {
			if(loops>0)diff+=(yold[i]-y[i])*(yold[i]-y[i]);
			yold[i]=y[i];
		}
		//if(loops>0&&diff<STOP_CRITERIA)break;
      	for(j=0;j<dim;j++) { 
	  		if(j==k) continue; // don't update query 
	  		row_ind=matrixind[j]; row_eval=Tmat[j]; isHom_r=isHom[j]; 
	  		double tmp=0;  
	  		
	  		for(i=0;i<nodeList[j]->deg;i++)  
	    		if(PPIflag || isHom_r[i]){ 
	    			tmp+=row_eval[i]*yold[row_ind[i]]; // weighted comb of neighbors
	    			//if(j==2)printf("%s(%d): %g*%g\n",nodeList[row_ind[i]]->name,nodeList[row_ind[i]]->id,row_eval[i],yold[row_ind[i]]);	
	    		} 
  			//y[j]=(alpha*tmp)+(1-alpha)*yorig[j];
  			//if(j==2)printf("%s: alpha*%g + %g = %g\n",nodeList[j]->name,tmp,yorig[j],(alpha*tmp)+yorig[j]);
  			y[j]=(alpha*tmp)+yorig[j];  // new y is old y + new weighted combination of neighbors
  		} 
		//printf("diff=%g\n",diff);
	}
	/*for(int i=0;i<3;i++)
		printf("Node %s Initial score=%g activation score %g\n",
			nodeList[tmpid[i]]->name,yorig[tmpid[i]],y[tmpid[i]]);fflush(stdout);
	*/
	//if(diff>1e-8)printf("\nDid not converge[delta=%g]\n",diff);
	//else printf("\nConverged after %d iterations (diff=%g, norm1=%g)\n",num_iter,delta,norm1(R_1,N));
		
	delete[] yold;delete[] yorig;
	
} 

void ZSR(){
	if(computed_PairWise)return;computed_PairWise=true;
	if(!allocated_PairWise){
		G0=new double[dim];
		PairWiseMat_help=new double*[dim];
		W_mul_G0=new double*[dim];
		aggregateMat=new double*[dim];
		PairWiseMat=new double*[dim];
		for(int i=0;i<dim;i++){
			PairWiseMat_help[i]=new double[dim];
			W_mul_G0[i]=new double[dim];
			aggregateMat[i]=new double[dim];
		}
	}
	allocated_PairWise=true;

	for(int i=0;i<dim;i++){
		G0[i]=0;
		for(int j=0;j<dim;j++){
			PairWiseMat_help[i][j]=0;
			aggregateMat[i][j]=0;
		}
		for(int j=0;j<nodeList[i]->deg;j++){
			PairWiseMat[i][matrixind[i][j]]=Tmatrix[i][j];	
			W_mul_G0[i][j]=Tmatrix[i][j];
			G0[i]+=Tmatrix[i][j];
		}G0[i]=1/(G0[i]+alpha);
	}
	
	//move on all rows in W_mul_G0 and multiple according to column index
	int ii;
	for(int i=0;i<dim;i++){
		for(int j=0;j<nodeList[i]->deg;j++){
			W_mul_G0[i][j]*=G0[matrixind[i][j]];
		}
	}
	double **tmp;	
	for(int loop=0;loop<10;loop++){
		mult(W_mul_G0,PairWiseMat,PairWiseMat_help,dim);
		SWAP(PairWiseMat,PairWiseMat_help,tmp);
		mat_plus(aggregateMat,PairWiseMat,dim);
	}
	//move on all rows in PairWiseMat and multiple according to row index
	for(int i=0;i<dim;i++){
		for(int j=0;j<dim;j++){
			PairWiseMat[i][j]=G0[i]*aggregateMat[i][j];
		}
	}
}
void diffuse_all_alg_help(int loopCntr,double *y,double *y_addition, int *prev_k,int k){
	#ifdef VERBOSE
	if(loopCntr%25==0){printf(".");fflush(stdout);}
	#endif
	int i;
	if(NRM_METHOD==0)normalize(Tmatrix,OPT2!=1&&OPT2!=8,k,*prev_k);*prev_k=k;
	diffuse(y,k,true,Tmatrix);
	if(OPT2==4 || OPT2==9){//delta_ppi
		diffuse(y_addition,k,false,Tmatrix);//Tmatrix_dppi
		for(i=0;i<dim;i++)y[i]-=y_addition[i];
	}
	if(OPT2==6){
		for(i=0;i<dim;i++)if(y[i]>=TINY)y[i]=-y[i];else y[i]=-999999;
	}
}



void diffuse_all_alg(int *loopCntr,double *y,double *y_addition,double *y_addition2, int *labels, int *prev_k,int k){
	if(OPT2==10){//ZSR
		ZSR();
		y=PairWiseMat[k];
	}else{
		if(OPT2==11){//PRANK
			biased_Prank(y,k);
		}
		else
			diffuse_all_alg_help(*loopCntr,y,y_addition,prev_k,k);(*loopCntr)++;
	}
	bool found=false;
	for(int i=0;i<dim;i++)if(nodeList[i]->spec==7202){y[i]=-1;found=true;}//9606
	//if(!found)my_exit("No HUMAN DATA\n");
	mergeSort(y,labels,dim);
	int i;
	if(OPT2==5 || OPT2==8 || OPT2==9){//Mutual algorithms: MHRP,MRP,MdHRP
		  		int numHom=0,numPPI=0;
		  		int spec=nodeList[k]->spec;
		  		double tmparr_hom[dim],tmparr_ppi[dim];
		  		int tmplabels_hom[dim],tmplabels_ppi[dim];
		  		int MHRP_PARAM_c=-1,MHRP_PARAM_h=-1;
		  		if(REL_TYPE==HOM || REL_TYPE==BOTH)MHRP_PARAM_h=MHRP_PARAM;
		  		if(REL_TYPE==PPI || REL_TYPE==BOTH)MHRP_PARAM_c=MHRP_PARAM;
		  		for(i=0;i<dim&& (numHom<MHRP_PARAM_h || numPPI<MHRP_PARAM_c);i++){
		  			node *nd=nodeList[ labels[dim-1-i]];
		  			if(nd->id==k || (nd->spec==spec && numPPI==MHRP_PARAM_c) 
		  				|| (nd->spec!=spec && numHom==MHRP_PARAM_h))continue;
		  			if( REL_TYPE==BOTH || 
		  				(REL_TYPE==HOM && nd->spec!=spec) ||
		  				(REL_TYPE==PPI && nd->spec==spec)){
		  					diffuse_all_alg_help(*loopCntr,y_addition,y_addition2,prev_k,nd->id);(*loopCntr)++;
		  				}
					
					if(nd->spec==spec){
						tmplabels_ppi[numPPI]=labels[dim-1-i];
						tmparr_ppi[numPPI++]=(y[dim-1-i]+y_addition[k])/2;
					}else{
						tmplabels_hom[numHom]=labels[dim-1-i];
						tmparr_hom[numHom++]=(y[dim-1-i]+y_addition[k])/2;
					}	
		  		}
		  		bool ismember;
		  		///////HOM
		  		if(REL_TYPE==HOM || REL_TYPE==BOTH){
			  		//for(int h=0;h<numHom;h++)printf("%g %d\n",tmparr_hom[h],tmplabels_hom[h]);printf("\n");
			  		mergeSort(tmparr_hom,tmplabels_hom,numHom,true);
			  		int cntr1=numHom;numHom=0;
			  		for(i=0;i<dim&&numHom<cntr1;i++){
			  			if(nodeList[labels[i]]->spec==spec)continue;
			  			ismember=false;
			  			for(int h=0;h<cntr1&&!ismember;h++)ismember=labels[i]==tmplabels_hom[h];
			  			if(!ismember)continue;
			  			labels[i]=tmplabels_hom[numHom];
			  			y[i]=tmparr_hom[numHom++];
			  		}
			  		if(numHom!=cntr1){printf("%d %d\n",numHom,cntr1);my_exit("MHRP:error\n");}
		  		}
		  		if(REL_TYPE==PPI || REL_TYPE==BOTH){
			  		///////PPI
			  		mergeSort(tmparr_ppi,tmplabels_ppi,numPPI,true);
			  		int cntr1=numPPI;numPPI=0;
			  		for(i=0;i<dim&&numPPI<cntr1;i++){
			  			if(nodeList[labels[i]]->spec!=spec)continue;
			  			ismember=false;
			  			for(int h=0;h<cntr1&&!ismember;h++)ismember=labels[i]==tmplabels_ppi[h];
			  			if(!ismember)continue;
			  			labels[i]=tmplabels_ppi[numPPI];
			  			y[i]=tmparr_ppi[numPPI++];
			  		}
			  		if(numPPI!=cntr1)my_exit("MHRP:error\n");
		  		}
	  	}	
	for(int i=0;i<dim;i++)if(nodeList[labels[i]]->spec==nodeList[k]->spec){y[i]=-1;found=true;}
}


void ranking(int ver) {   
	  int i,k,loopCntr=0,labels[dim];
	  char fname[NAME_LEN];
	  FILE *fpDtlsC,*fpValC,*fpLabC,*fpDtlsH,*fpValH,*fpLabH;
	  bool has_ppi=false,has_hom=false;
	  /*Check if configuration was already tested*/
	  sprintf(fname,"%s/dtls/ranks.ppi.%d",postfix,ver);
	  FILE *tmpF=fopen(fname,"r");if(tmpF!=NULL){fclose(tmpF);has_ppi=true;}
	  sprintf(fname,"%s/dtls/ranks.hom.%d",postfix,ver);
	  tmpF=fopen(fname,"r");if(tmpF!=NULL){fclose(tmpF);has_hom=true;}
	  /*if(REL_TYPE==PPI&&has_ppi || REL_TYPE==HOM&&has_hom || REL_TYPE==BOTH&&has_ppi&&has_hom){
	  		printf("Configuration %d already computed\n",ver);
	  		return;
	  }*/
	  sprintf(fname,"%s/dtls/ranks.hom.%d",postfix,ver);
	  //DLS(fname)
	  if(REL_TYPE==PPI || REL_TYPE==BOTH){
	  	sprintf(fname,"%s/dtls/ranks.ppi.%d",postfix,ver);
	  	fpDtlsC=fopen(fname,"w");if(fpDtlsC==NULL){
	  		printf("Can't write to %s\n",fname);
	  		my_exit("IO error 1\n");
	  	}
  	  	sprintf(fname,"%s/dtls/vals.ppi.%d",postfix,ver);
  	  	fpValC=fopen(fname,"w");if(fpValC==NULL)my_exit("IO error 1\n");
	  	sprintf(fname,"%s/dtls/label.ppi.%d",postfix,ver);
  	  	fpLabC=fopen(fname,"w");if(fpLabC==NULL)my_exit("IO error 1\n");
	  }
	  if(REL_TYPE==HOM || REL_TYPE==BOTH){
	  	sprintf(fname,"%s/dtls/ranks.hom.%d",postfix,ver);
	  	fpDtlsH=fopen(fname,"w");if(fpDtlsH==NULL)my_exit("IO error 1\n");
	  	sprintf(fname,"%s/dtls/vals.hom.%d",postfix,ver);
  	  	fpValH=fopen(fname,"w");if(fpValH==NULL)my_exit("IO error 1\n");
	  	sprintf(fname,"%s/dtls/label.hom.%d",postfix,ver);
  	  	fpLabH=fopen(fname,"w");if(fpLabH==NULL)my_exit("IO error 1\n");
	  }
	  double *y=new double[dim];double *y_addition=new double[dim];
	  double *y_unsorted=new double[dim];double *y_addition2=new double[dim];
	  int prev_k=-1;
	  for(map<int,bool>::iterator kk=Qmap.begin();kk!=Qmap.end();kk++) 
	  {
		  	k=kk->first;
		  	//printf("Q=%d\n",k);
		  	diffuse_all_alg(&loopCntr,y,y_addition,y_addition2,labels,&prev_k,k);
		  	for(i=0;i<dim;i++)
				y_unsorted[labels[i]]=y[i];
				
		  	if(REL_TYPE==HOM || REL_TYPE==BOTH)calc_performance(k,fpDtlsH,fpValH,fpLabH,&(nodeList[k]->GSH),y,labels,y_unsorted,true);
			if(REL_TYPE==PPI || REL_TYPE==BOTH)calc_performance(k,fpDtlsC,fpValC,fpLabC,&(nodeList[k]->GSC),y,labels,y_unsorted,false);	

			break;
	  }
	  computed_PairWise=false;
	  if(REL_TYPE==HOM || REL_TYPE==BOTH){fclose(fpValH);fclose(fpLabH);}
	  if(REL_TYPE==PPI || REL_TYPE==BOTH){fclose(fpValC);fclose(fpLabC);}
	  #ifdef VERBOSE
	  printf("\n");
	  #endif
	  delete[] y;delete[] y_addition;delete[]y_unsorted;
} 

void biased_Prank(double *y,int k){
	nlist2 arr1,arr2;
	double sum[dim];
	int cntr2=0,cntr1=0,first_spec=nodeList[0]->spec;
	for(int i=0;i<dim;i++){
		sum[i]=0;
		if(nodeList[i]->spec==first_spec)arr1[i]=cntr1++;
		else arr2[i]=cntr2++;
		for(int k=0;k<nodeList[i]->deg;k++){
			if(!isHom[i][k])sum[i]+=matrix[i][k];
		}
	}
	
	if(!computed_PairWise){
		double tot_hom_sum=0;int N=cntr1*cntr2;
		printf("Calculating biased_Prank for %d pairs\n",N);fflush(stdout);
		double *R_1=new double[N],
			*R_2=new double[N],
			*diffR=new double[N],
			*homVec=new double[N];
		for(int i=0;i<N;i++)homVec[i]=0;
		for(nlist2::iterator i1=arr1.begin();i1!=arr1.end();i1++){
			int ii=i1->second,i=i1->first;
			for(nlist2::iterator j1=arr2.begin();j1!=arr2.end();j1++){
				int jj=j1->second,j=j1->first;
				int ind=ii*cntr2+jj;
				homVec[ind]=0;
				int k=ismember(j,matrixind[i],nodeList[i]->deg);
				if(k!=-1){
					homVec[ind]= tf2(matrix[i][k],MAXh);
					if(homVec[ind]<0 || fpclassify(homVec[ind])==FP_NAN)my_exit("Negative\n");
					//printf("%s-%s %g %g\n",nodeList[i]->name,nodeList[j]->name,matrix[i][k],homVec[ind]);
					tot_hom_sum+=homVec[ind];
				}
			}
		}
		for(int i=0;i<N;i++){
			homVec[i]/=tot_hom_sum;
			R_1[i]=homVec[i];R_2[i]=0;//1./N
		}
		//FILE *ftmp=fopen("tmp","w");
		double RWalpha=0.6;
		double EPS=1e-10,delta=99.;
		int MAX_ITER=750,num_iter=0;
		do{
			//if(num_iter%10==1){
				printf("[%d:]%g, \n",num_iter,delta);fflush(stdout);
			//}
			for(nlist2::iterator i1=arr1.begin();i1!=arr1.end();i1++){
				int ii=i1->second,i=i1->first;
				for(nlist2::iterator j1=arr2.begin();j1!=arr2.end();j1++){
					int jj=j1->second,j=j1->first;
					int ind=ii*cntr2+jj;R_2[ind]=0;
					double tmp1=0;
					for(int k1=0;k1<nodeList[i]->deg;k1++){
						if(isHom[i][k1])continue;
						for(int k2=0;k2<nodeList[j]->deg;k2++){
							if(isHom[j][k2])continue;
							R_2[ind]+=(matrix[i][k1]*matrix[j][k2]*
								R_1[arr1[matrixind[i][k1]]*cntr2+arr2[matrixind[j][k2]]])/
								(sum[matrixind[i][k1]]*sum[matrixind[j][k2]]);
							/*if(num_iter==0)fprintf(ftmp,"%d %d %g\n",ind,arr1[matrixind[i][k1]]*cntr2+arr2[matrixind[j][k2]],
								matrix[i][k1]*matrix[j][k2]/
								(sum[matrixind[i][k1]]*sum[matrixind[j][k2]]));
							printf("R2[%s-%s]+=W[%s-%s]*w[%s-%s]*R1[%s-%s] = %g * %g * %g\n",
								nodeList[i]->name,nodeList[j]->name,
								nodeList[i]->name,nodeList[matrixind[i][k1]]->name,
								nodeList[j]->name,nodeList[matrixind[j][k2]]->name,
								nodeList[matrixind[i][k1]]->name,nodeList[matrixind[j][k2]]->name,
								matrix[i][k1],matrix[j][k2],
								R_1[arr1[matrixind[i][k1]]*cntr2+arr2[matrixind[j][k2]]]);*/
						}
					}
					R_2[ind]=RWalpha*R_2[ind]+(1.-RWalpha)*homVec[ind];
					/*printf("%s-%s: R1=%g, R_2=%g(%g ,%g ,%g,%d %d %d %d)\n",
						nodeList[i]->name,nodeList[j]->name,R_1[ind],
						R_2[ind],homVec[ind],sum[i],sum[j],i,ii,j,jj);FLS;*/
					//if(get_abs(tmp1-sum[i]*sum[j])>1e-10)my_exit("error\n");
				}
			}//if(num_iter==0)fclose(ftmp);
			normalize_vec_nrm1(R_2,N);
			vec_minus(R_1,R_2,diffR,N);
			delta=norm2(diffR,N);
			//for(int i=0;i<N;i++)R_1[i]=R_2[i];
			memcpy(R_1,R_2,sizeof(double)*N);
			num_iter++;
		}while(delta>EPS && num_iter<MAX_ITER);
		normalize_vec_nrm1(R_1,N);
		#ifdef VERBOSE
		if(delta>EPS)printf("\nDid not converge\n");
		else printf("\nConverged after %d iterations (diff=%g, norm1=%g)\n",num_iter,delta,norm1(R_1,N));
		#endif
		PRank_vec=R_1;
		delete[]R_2;delete[]homVec;delete[]diffR;
	}

	computed_PairWise=true;allocated_PairWise=true;
	int spec=nodeList[k]->spec;
	if(spec==first_spec){
		for(int i=1;i<dim;i++){
			if(nodeList[i]->spec==spec)y[i]=-1;
			else {y[i]=PRank_vec[arr1[k]*cntr2+arr2[i]];}//DLF(y[i]);}
		}
	}
	else{
		for(int i=1;i<dim;i++){
			if(nodeList[i]->spec==spec)y[i]=-1;
			else {y[i]=PRank_vec[cntr2*arr1[i] + arr2[k]];}//DLF(y[i]);}
		}
	}
}


void calc_performance(int k,FILE *fp_dtls,FILE *fp_vals,FILE *fp_labs,nlist *GS,
	double *y,int *labels,double *y_unsorted,bool hom){
	    if(GS->size()==0)return;
	    int i=0,spec=nodeList[k]->spec; 
	    node *nd;int PRINT_LIMIT=10;
	    int cntr=0,num_false=0;
	    double val_arr[MAX_ROC_LIMIT+200];
	    short label_arr[MAX_ROC_LIMIT+200];
	    //printf("calc_performance: %s\n", nodeList[k]->name);fflush(stdout);
		if(OPT2!=11)fprintf(fp_dtls,">%s(",nodeList[k]->name);
	    map<int ,bool>visited;
	    for(nlist::iterator ii=GS->begin();ii!=GS->end();ii++){
  				if(OPT2!=11)fprintf(fp_dtls,"%s-%d, ",ii->first->name,ii->second);
  				visited[ii->first->id]=false;
	    }if(OPT2!=11)fprintf(fp_dtls,")\n");
	    map<int,bool>PPImap;
	    if(!hom){
	    	for(int j=0;j<nodeList[k]->deg;j++)
	    	{
		    	if(!isHom[k][j])PPImap[matrixind[k][j]]=false;
	    	}
	    }
	    while(i<dim&& num_false<MAX_ROC_LIMIT){
	    	nd=nodeList[labels[dim-1-i]];
	    	if(nd->id==k){i++;continue;}
	    	if(hom && nd->spec!=spec || !hom&& nd->spec==spec){
				if(num_false<PRINT_LIMIT){
					if(OPT2!=11)
						fprintf(fp_dtls,"%s\t%g\n",nd->name,y[dim-1-i]);
					else
						fprintf(fp_dtls,"%s\t%s\t%g\n",nodeList[k]->name,nd->name,y[dim-1-i]);
				}
	  			val_arr[cntr]=y[dim-1-i];
	  			if(GS->find(nd)!=GS->end()){
	  				label_arr[cntr]=(*GS)[nd];
	  				visited[nd->id]=true;
	  				if(OPT2!=11)fprintf(fp_dtls,"GS: %d\n",cntr);
	  			}else{
	  				if(PPImap.find(nd->id)!=PPImap.end()){
	  					label_arr[cntr]=2;
	  				}
	  				else label_arr[cntr]=0;
	  			}
	  			if(PPImap.find(nd->id)!=PPImap.end())PPImap[nd->id]=true;
	  			if(label_arr[cntr]!=1)num_false++;
	  			cntr++;
	    	}
	    	i++;	
	    }
	    fprintf(fp_vals,"%s ",nodeList[k]->name);
	    fprintf(fp_labs,"%s ",nodeList[k]->name);
	    for(i=0;i<cntr;i++){
	   		fprintf(fp_vals,"%g ",val_arr[i]);
	   		fprintf(fp_labs,"%d ",label_arr[i]);
	   	}
	   	for(nlist::iterator ii=GS->begin();ii!=GS->end();ii++){
  				if(!visited[ii->first->id]){
  					fprintf(fp_vals,"%g ",y_unsorted[ii->first->id]);
	   				fprintf(fp_labs,"%d ",ii->second);
	   				if(PPImap.find(ii->first->id)!=PPImap.end())PPImap[ii->first->id]=true;
	   				//if(ii->second==2){
	   					//printf("Warning: %s 2nd best in the end\n",nodeList[k]->name);
	   				//}//  
					if(OPT2==11)fprintf(fp_dtls,"%s\t%s\t%g\n",nodeList[k]->name,ii->first->name,y_unsorted[ii->first->id]);
  				}
	    }
	    //print the rest for PPI
	    if(!hom){
		    for(map<int,bool>::iterator ii=PPImap.begin();ii!=PPImap.end();ii++){
		    	if(!ii->second){
	  					fprintf(fp_vals,"%g ",y_unsorted[ii->first]);
		   				fprintf(fp_labs,"2 ");
		   		}
		    }
	    }
	   	fprintf(fp_vals,"\n");fprintf(fp_labs,"\n");
}



void calc_emp_pval(double ***rank1,double ***rank_sig1,double **median_sig1,bool mixInQset){
	int i,k,qnum,loopCntr=0,labels[dim];
	node *nd;
	double *y=new double[dim];
	double *y_addition=new double[dim];
	double *y_addition2=new double[dim];
	double *y_unsorted=new double[dim];
	double *mean_rank=new double[dim];
	double *mean_rank_rnd=new double[dim];
	double **tot_rank=new double*[Qmap.size()+1];
	double **tot_rank_unsorted=new double*[Qmap.size()+1];
	double **tot_rank_rnd=new double*[Qmap.size()+1];
	int **tot_labels=new int*[Qmap.size()+1];
	double *median_sig=new double[dim];
	double **rank_sig=new double*[Qmap.size()+1];
	
	for(int q=0;q<(Qmap.size()+1);q++){
		tot_rank[q]=new double[dim];
		tot_rank_unsorted[q]=new double[dim];
		tot_rank_rnd[q]=new double[dim];
		tot_labels[q]=new int[dim];
		rank_sig[q]=new double[dim];
		for(int i=0;i<dim;i++)rank_sig[q][i]=0;
	}
	for(int i=0;i<dim;i++)median_sig[i]=0;
	
	int tmpspec=nodeList[Qmap.begin()->first]->spec;
	int actual_dim=0;
	bool participating[dim];
	for(int k1=0;k1<dim;k1++){
		participating[k1]= REL_TYPE==BOTH || 
		   (REL_TYPE==PPI && nodeList[k1]->spec==tmpspec) ||
		   (REL_TYPE==HOM && nodeList[k1]->spec!=tmpspec);
		if(participating[k1])actual_dim++;
	}
	#ifdef VERBOSE
	printf("Num participating: %d",actual_dim);
	#endif
	int NUM_ITER1=NUM_ITER;
	if(OPT2==0 || OPT2==6 || OPT2==7)NUM_ITER1=2;
	if(OPT2==11)NUM_ITER1=1;
	//////////////////
	NUM_ITER1=1;// DO NOT PERFOEM PERMUTATION TEST
	//////////////////
	for (int iter=0;iter<NUM_ITER1;iter++){
		#ifdef VERBOSE
		printf("=====Iteration no.%d=======\n",iter);fflush(stdout);
		#endif
		if(iter==1){shuffle(200.,mixInQset);shuffleQ(200.);
			#ifdef VERBOSE 
				printf(" ");
			#endif
		}
		if(iter>1){shuffle(50.,mixInQset);shuffleQ(50.);
			#ifdef VERBOSE 
				printf(" ");
			#endif
		}
		if(iter>0&&NRM_METHOD==1)normalize(Tmatrix,OPT2!=1&&OPT2!=8,-1,-1);
		qnum=0;int prev_k=-1;
		for(map<int,bool>::iterator kk=Qmap.begin();kk!=Qmap.end();kk++) 
	    {
	    		k=kk->first;
			  	diffuse_all_alg(&loopCntr,y,y_addition,y_addition2,labels,&prev_k,k);
			  	for(i=0;i<dim;i++){y_unsorted[labels[i]]=y[i];}
			  	if(iter==0){
			  		memcpy(tot_rank[qnum],y,dim*sizeof(double));
			  		memcpy(tot_labels[qnum],labels,dim*sizeof(int));
			  		memcpy(tot_rank_unsorted[qnum],y_unsorted,dim*sizeof(double));
			  	}
			  	else{
			  		 memcpy(tot_rank_rnd[qnum],y_unsorted,dim*sizeof(double));
			  		 int k2=0,k3=0; //k2 is a counter over all enrties, k3 is counter over relevant ones (PPI/HOM)
					 for(int k1=0;k1<dim;k1++){
					 	while(k2<dim&&y[k2]<tot_rank[qnum][k1]){
					 			if(participating[labels[k2]])k3++;k2++;
					 	}
					 	rank_sig[qnum][tot_labels[qnum][k1]]+=actual_dim-k3;
					 }
			  	}
			  	qnum++;
	   }
	    if(iter==0){
	   		double tmparr[qnum];
		  	for(i=0;i<dim;i++){
		  		if(participating[i]){
		  			for(int q=0;q<(qnum);q++)tmparr[q]=tot_rank_unsorted[q][i];
		  			mean_rank[i]=median(tmparr,qnum);
		  		}else mean_rank[i]=-1;
		  	}
		  	#ifdef VERBOSE 
		  		printf("Max median score (real): %g\n",get_max(mean_rank,dim));
		  	#endif
		}
		else{
			double tmparr[qnum];
			for(i=0;i<dim;i++){
				if(participating[i]){
					for(int q=0;q<qnum;q++)tmparr[q]=tot_rank_rnd[q][i];
					mean_rank_rnd[i]=median(tmparr,qnum);
				}else mean_rank_rnd[i]=-1;
			}
			mergeSort(mean_rank_rnd,labels,dim);
			#ifdef VERBOSE
			printf("Max median score (sim): %g\n",mean_rank_rnd[dim-1]);
			#endif
			for(i=0;i<dim;i++)
				if(participating[i]){
					for(int j=0;j<dim && mean_rank[i]<=mean_rank_rnd[dim-j-1];j++)
						if(participating[labels[dim-j-1]])median_sig[i]++;
				}
	   }
	    #ifdef VERBOSE
	    printf("\n");
	    #endif
	}computed_PairWise=false;
	if(NUM_ITER1>1){
		for(int i=0;i<dim;i++){
	 	if(participating[i]){
	 		for(int q=0;q<(qnum);q++){
	 			if(rank_sig[q][i]==0)rank_sig[q][i]=1./((double)(actual_dim*(NUM_ITER1-1))+1);
	 			else rank_sig[q][i]/=((double)(actual_dim*(NUM_ITER1-1)));
	 		}
	 		if(median_sig[i]==0)median_sig[i]=1./((double)(actual_dim*(NUM_ITER1-1))+1);
	 		else median_sig[i]/=((double)(actual_dim*(NUM_ITER1-1)));
	 	}
	 	else{
	 		for(int q=0;q<(qnum);q++)rank_sig[q][i]=99;
	 		median_sig[i]=99;
	 	}
	 } 
	}
	for(int q=0;q<qnum;q++){
		delete[]tot_rank[q];
		delete[]tot_rank_rnd[q];
		delete[]tot_labels[q];
	 }
	delete[] y;delete[] y_addition;delete[]tot_rank;
	delete[]tot_rank_rnd;
	delete[]tot_labels;delete[]y_unsorted;
	delete[]mean_rank_rnd;delete[]mean_rank;
	delete[] y_addition2;
	*rank_sig1=rank_sig;*median_sig1=median_sig;
	*rank1=tot_rank_unsorted;
}


void emp_pval(){
	#ifdef VERBOSE
	printf("emp-pval dist\n");fflush(stdout);
	#endif
	double **rank_sig,**rank,*median_sig;
	calc_emp_pval(&rank,&rank_sig,&median_sig,true);
	char fname[256];
	sprintf(fname,"%s/emp_dist",postfix);
	FILE *fp=fopen(fname,"w");
	int qnum=0;bool posGS;
	for(map<int,bool>::iterator kk=Qmap.begin();kk!=Qmap.end();kk++){
		int k=kk->first;
		for(int i=0;i<dim;i++){
			if(rank_sig[qnum][i]!=99){
				posGS=(nodeList[k]->GSH.find(nodeList[i])!=nodeList[k]->GSH.end() && 
							nodeList[k]->GSH[nodeList[i]]==1) || 
					  (nodeList[k]->GSC.find(nodeList[i])!=nodeList[k]->GSC.end() && 
							nodeList[k]->GSC[nodeList[i]]==1);
				fprintf(fp,"%g %d\n",rank_sig[qnum][i],posGS);
			}
		}
		qnum++;
	}fclose(fp);
	for(int q=0;q<qnum;q++){
		delete[]rank_sig[q];delete[]rank[q];
	}
	delete[] rank_sig;delete[] median_sig;delete[]rank;
} 


void clean_grph(){
  for(int i=0;i<dim;i++){
  	if(nodeList[i]->degOut>0)delete[]matrixind_out[i];
  	if(nodeList[i]->deg==0)continue;
  	delete[]matrix[i];delete[]matrixind[i];
  	delete[]isHom[i];
  }
  delete[]matrix;delete[]matrixind;
  delete[]isHom;delete[]matrixind_out;	
}

void clean(){
  #ifdef VERBOSE
  printf("Clean memory and exit...\n");
  #endif
  for(int i=0;i<dim;i++)if(nodeList[i]->deg>0)delete[]Tmatrix[i];
  delete[]Tmatrix;
  clean_grph();
  for(Xref::iterator ii=nodeXref.begin();ii!=nodeXref.end();ii++)
  	{delete(ii->second);delete(ii->first);}	
  if(allocated_PairWise){
	  allocated_PairWise=false;
	  if(OPT2==10){
		for(int i=0;i<dim;i++){
			delete[]PairWiseMat_help[i];
			delete[]W_mul_G0[i];
			delete[]aggregateMat[i];
		}
		delete[]G0;
		delete[]PairWiseMat_help;
		delete[]W_mul_G0;
		delete[]aggregateMat;
		delete[]PairWiseMat;
	  }
	  else delete[]PRank_vec;
	}
	IDPROT=0;dim=0;nodeXref.clear();tot_ppi=0;
}


void run_gs(char *homDir){
  printf("run_gs\n");
  double GRID[]={-5,-2,0,2,5};int GRID_SZ=5;
  double ALPHA_GRID[]={0.3,0.95};//,1.6};
  int ALPHA_GRID_SZ=2;
  
  char fH[MAX_SPEC][NAME_LEN],fC[MAX_SPEC][NAME_LEN],Qfile[NAME_LEN],
  	GSCfile[NAME_LEN],GSHfile[NAME_LEN];
  int sH[MAX_SPEC][2],sC[MAX_SPEC][2];
  int nH,nC,ver=0,win_c=-1;
  char fname[NAME_LEN];
  if(OPT2==0 || OPT2==6 || OPT2==7){MAX_LOOP=1;}
  if(TASK==TASK_Q)sprintf(fname,"%s/config_q",homDir);
  //else if(dataset==NMD && REL_TYPE==PPI)sprintf(fname,"%s/config_large_scale",homDir);
  	//		else 
  sprintf(fname,"%s/config",homDir);
  load_config(fname,fH,&nH,sH,fC,&nC,sC,Qfile,GSCfile,GSHfile);
  load_matrix(fC,sC,nC,fH,sH,nH,OPT2!=1&&OPT2!=8);
  
  char REL_TYPE_str[32];
  if(REL_TYPE==HOM || REL_TYPE==BOTH)sprintf(REL_TYPE_str,"hom");
  else sprintf(REL_TYPE_str,"ppi");
  if(TASK==TASK_EMP)win_c=read_win(postfix,REL_TYPE_str);
  if(TASK==TASK_Q){
  	#ifdef VERBOSE
  	printf("TASK is query analysis: ");fflush(stdout);
  	#endif
  	win_c=read_win(postfix,REL_TYPE_str);
	if(dataset==MRF)load_query2(Qfile,false);//for test data
	else load_query(Qfile);
  }
  else{
  	#ifdef VERBOSE_inp
  	printf("Config: %d PPI files and %d homology files\n",nC,nH);fflush(stdout);
  	#endif
  	if(OPT2!=1 && (REL_TYPE==PPI || REL_TYPE==BOTH) )loadGS(GSCfile,false);
  	if(OPT2!=4 && (REL_TYPE==HOM || REL_TYPE==BOTH) )loadGS(GSHfile,true);
  }
  int strt=0;if(OPT2==1 || OPT2==8)strt=GRID_SZ-1;
  bool TASK_EMP_Q=TASK==TASK_EMP||TASK==TASK_Q;
 
  
  for(int i=0;i<GRID_SZ;i++){ 
  	if(GRID[i]<0)continue;
  	sigmaH=pow(10,GRID[i]); //sigma starts from zero
  	for(int j=strt;j<GRID_SZ;j++){
  		if(GRID[j]<0)continue; //sigma starts from zero
  		sigmaC=pow(10,GRID[j]);
  		for(int k=strt;k<GRID_SZ;k++){
  			rho=pow(5,GRID[k]);
  			for(int l=0;l<ALPHA_GRID_SZ;l++){
  				alpha=ALPHA_GRID[l];
  				if(OPT2==0 || OPT2==6 || OPT2==7){rho=1;alpha=0;}      
  				printf("ver=%d sigmaH=%g sigmaC=%g rho=%g alpha=%g\n",
					ver, sigmaH,sigmaC,rho,alpha);//ver++;continue;
  				if( (!TASK_EMP_Q || (TASK_EMP_Q && ver==win_c)) && NRM_METHOD==1 && OPT2!=11)
  					normalize(Tmatrix,OPT2!=1&&OPT2!=8,-1,-1);   
  				if(!TASK_EMP_Q)ranking(ver);
  				else if(ver==win_c){if(TASK==TASK_EMP)emp_pval();else analyze_q(-1);}
  				if(OPT2==0 || OPT2==6 || OPT2==7 || OPT2==11 || (TASK_EMP_Q&&ver==win_c)){clean();exit(0);}
  				ver++;
  			}
  		}
  	}
  }
  clean();	
}

void analyze_gs(char *homDir,int selected_ver){
  //double GRID[]={-4,-2,-1,0,1,2,4};int GRID_SZ=7;
  double GRID[]={-5,-2,0,2,5};int GRID_SZ=5;
  double ALPHA_GRID[]={0.3,0.95};//,1.6};
  int ALPHA_GRID_SZ=2;//1;
  
  char fH[MAX_SPEC][NAME_LEN],fC[MAX_SPEC][NAME_LEN],Qfile[NAME_LEN],
  	GSCfile[NAME_LEN],GSHfile[NAME_LEN],cmd[NAME_LEN];
  int sH[MAX_SPEC][2],sC[MAX_SPEC][2];
  int nH,nC,ver=0,win_c=-1;
  char fname[NAME_LEN];
  if(OPT2==0 || OPT2==6 || OPT2==7){MAX_LOOP=1;}
  sprintf(fname,"%s/config",homDir);
  load_config(fname,fH,&nH,sH,fC,&nC,sC,Qfile,GSCfile,GSHfile);
  load_matrix(fC,sC,nC,fH,sH,nH,OPT2!=1&&OPT2!=8);
  load_query2(GSHfile,true);//For GS data to make reports (incl stat sig)
  int * best_cnfg=load_best_configs(postfix);
  int strt=0;if(OPT2==1 || OPT2==8)strt=GRID_SZ-1;
  map<int,bool>Qsave;bool first=true;
  for(map<int,bool>::iterator kk=Qmap.begin();kk!=Qmap.end();kk++)Qsave[kk->first]=kk->second;
  Qmap.clear();bool cont=true;
  for(int i=0;cont&&i<GRID_SZ;i++){
  	if(GRID[i]<0)continue;
  	sigmaH=pow(10,GRID[i]); //sigma starts from zero
  	for(int j=strt;cont&&j<GRID_SZ;j++){
  		if(GRID[j]<0)continue; //sigma starts from zero
  		sigmaC=pow(10,GRID[j]);
  		for(int k=strt;cont&&k<GRID_SZ;k++){
  			rho=pow(5,GRID[k]);
  			for(int l=0;cont&&l<ALPHA_GRID_SZ;l++){
  				alpha=ALPHA_GRID[l];
  				if(OPT2==0 || OPT2==6 || OPT2==7){rho=1;alpha=0;}      
  				if(NRM_METHOD==1)normalize(Tmatrix,OPT2!=1&&OPT2!=8,-1,-1);   
				for(map<int,bool>::iterator kk=Qsave.begin();kk!=Qsave.end();kk++)
					if(best_cnfg[kk->first]==ver)Qmap[kk->first]=kk->second;
				if(!Qmap.empty() && ver==selected_ver){
					/*if(!first){
						clean();//Need to re-read data due to shuffling
						load_matrix(fC,sC,nC,fH,sH,nH,OPT2!=1&&OPT2!=8);
						load_query2(GSHfile,true);
						best_cnfg=load_best_configs(postfix);
						Qsave.clear();
						for(map<int,bool>::iterator kk=Qmap.begin();kk!=Qmap.end();kk++)Qsave[kk->first]=kk->second;
						Qmap.clear();
						for(map<int,bool>::iterator kk=Qsave.begin();kk!=Qsave.end();kk++)
							if(best_cnfg[kk->first]==ver)Qmap[kk->first]=kk->second;
					}*/
					printf("Analyzing %d queries from config %d\n",Qmap.size(),ver);
					analyze_q(ver);first=false;
				}Qmap.clear();
  				if(OPT2==0 || OPT2==6 || OPT2==7 || OPT2==11)cont=false;
  				ver++;
  			}
  		}
  	}
  }
  /*sprintf(cmd,"cat %s/query.tmpsig*.res > %s/query.sig-gs.res",postfix,postfix);system(cmd);
  sprintf(cmd,"rm %s/query.tmpsig*.res",postfix);system(cmd);
  sprintf(cmd,"cat %s/query.tmprank*.res > %s/query.rank-gs.res",postfix,postfix);system(cmd);
  sprintf(cmd,"rm %s/query.tmprank*.res",postfix);system(cmd);*/
  clean();	
}

void analyze_output(){
	double ***mat_val; short ***mat_label;int *npos;int **mat_len;char **names;
	int num_config,num_samples;int fold=20;//percent (<=> 5-fold)
	int szarr[]={500,50,5};char task_str[63];
	if(TASK==TASK_an_OUTPUT_EIG)szarr[0]=-50; 
	if(REL_TYPE==PPI)sprintf(task_str,"ppi");else sprintf(task_str,"hom");
	int flist[135];
	//load_dat_nt(&mat_val,&mat_label,&mat_len,&num_config,&num_samples,
	//	&npos,&names,task_str,szarr[0]+50,flist);
	load_dat(&mat_val,&mat_label,&mat_len,&num_config,&num_samples,
		&npos,&names,task_str,szarr[0]+50);
	if(TASK==TASK_an_OUTPUT_EIG){
		printf("----------------\nABS 500\n");
		analyze_CV_abs(fold,task_str,mat_val,mat_label,npos,names,num_config,
			num_samples,500,1,mat_len);
		printf("----------------\nREL 500\n");
		analyze_CV_rel(fold,task_str,mat_val,mat_label,npos,num_config,
			num_samples,500,1,mat_len);
	}
	else{
		for(int i=0;i<3;i++){
			for(int flag=0;flag<2;flag++){//stringent / permissive negative set
				if(flag==0 || flag==1 && i==0){
					printf("----------------\nABS %d flag=%d\n",szarr[i],flag);
					analyze_CV_abs(fold,task_str,mat_val,mat_label,npos,names,num_config,
						num_samples,szarr[i],flag,mat_len);
					printf("----------------\nREL %d flag=%d\n",szarr[i],flag);
					analyze_CV_rel(fold,task_str,mat_val,mat_label,npos,num_config,
						num_samples,szarr[i],flag,mat_len);
					
					analyze_CV_abs_nt(fold,task_str,mat_val,mat_label,npos,names,num_config,
						num_samples,szarr[i],flag,mat_len,flist);
					printf("----------------\nREL %d flag=%d\n",szarr[i],flag);
					analyze_CV_rel_nt(fold,task_str,mat_val,mat_label,npos,names,num_config,
						num_samples,szarr[i],flag,mat_len);					
				}
			}
		}
	}
	#ifdef VERBOSE
	printf("Clean memory and exit...\n");
	#endif
	for (int i=0;i<num_config;i++){
		for(int j=0;j<num_samples;j++){
			delete[] mat_val[i][j];
			delete[] mat_label[i][j];
		}
		delete[] mat_val[i];
		delete[] mat_label[i];	
		delete[] mat_len[i];
	}
	delete[] mat_val;
	delete[] mat_label; 
	delete[] mat_len;
	delete[] npos;
}





int main(int argc, char *argv[]) 
{ 
	char *libs[]={"smp", "only_hom", "all","all-deg","delta_ppi","MHRP","smp-abs",
		"PSIBLAST","MRP","MdHRP","ZSR","PRANK","temp"};
  	//#<home folder> <alg variant> <DB> <TASK> <PPI/HOM/BOTH>
  	OPT2=atoi(argv[2]);dataset=atoi(argv[3]);
  	sprintf(postfix,"%s/%s",argv[1],libs[OPT2]);
  	#ifdef VERBOSE
  	printf("home: %s\n",postfix);
  	#endif
  	TASK=atoi(argv[4]);//0-TASK_GS, 1-TASK_an_OUTPUT, 2-TASK_Q, 3-TASK_EMP, 4-TASK_CLEAN
  	REL_TYPE=atoi(argv[5]);//0-ppi,1=hom,2-both
  	if(argc>=7)Qalg_FLAG=atoi(argv[6]);// 0 - generate NBLAST output, x>0 -  top x candidates
  	//write_config();
  	if(TASK==TASK_CLEAN){
  		if(REL_TYPE==HOM || REL_TYPE==BOTH)clean_dir(postfix,"hom");
  		if(REL_TYPE==PPI || REL_TYPE==BOTH)clean_dir(postfix,"ppi");
  		exit(0);
  	}
  	if(REL_TYPE==PPI && (OPT2==1||OPT2==8) || REL_TYPE==HOM && (OPT2==4||OPT2==9)){
  		printf("Irrelevant task\n");
  		exit(0);
  	}
  	//if(TASK==TASK_Q)ONLY_INTERCATING_PROT=false;
  	#ifdef VERBOSE
  	printf("Task -- %d (%d)\n",TASK,REL_TYPE);fflush(stdout);
  	#endif
	switch (TASK){
		case TASK_ANALYZE_GS:
			if(argc<8)my_exit("Need to provide config ID\n");
			analyze_gs(argv[1],atoi(argv[7]));break;
		case TASK_an_OUTPUT:
			analyze_output();break;
		case TASK_an_OUTPUT_EIG:
			analyze_output();break;
		case TASK_RUN_EIG:
			eig_ppi(argv[1]);break;
		case TASK_GS:
			run_gs(argv[1]);break;
		case TASK_Q:
			run_gs(argv[1]);break;
		default:
			my_exit("Ilegal option\n");break;
	}
} 





















