#ifndef UTIL_CPP_
#define UTIL_CPP_
#include "util.h"

double lchoose( int n, int k )
{
  int i, min_k;
  double sum;

  if( (k==0) || (k==n) )
        return 0;

  if( (k<0) || (k>n) ) {
        fprintf (stderr,"Error! k=%d, n=%d\n", k, n);
        exit (1);
  }

  if( (k>10) && ((n-k)>10) ) {
        /* Use Stirling approximation */
        sum = n*log(n)-k*log(k)-(n-k)*log(n-k);
        sum -= 0.5*(log(k)+log(n-k)-log(n));
        sum -= LOG2P;
  } else {
        sum = log((double)n);
        min_k = (k>n/2) ? (n-k) : k;
        for( i=2 ; i<=min_k ; i++ )
          sum += log(((double)(n+1-i)) / ((double)i));
  }

  return sum;
}

double enrich_prob( int stat[6] )
{
  int i1, i2, i3;
  int hits, size, bg1, bg2, bg3, bg_size, bg0;
  int max;
  double logAll;
  double prob;

  hits = stat[0];
  size = stat[1];
  bg_size = stat[2];
  bg1 = stat[3];
  bg2 = stat[4];
  bg3 = stat[5];
  bg0 = bg_size-bg1-bg2-bg3;
  //if( hits<2 )return 1;
  prob = 0.;
  if( size>bg3 ) {
    max = bg3*3;
    if( (size-bg3)>bg2 ) {
      max += bg2*2;
	  if( (size-bg3-bg2)>bg1 )
		max += bg1;
	  else
		max += (size-bg3-bg2);
    } else
      max += 2*(size-bg3);
  } else
    max = 3*size;

  logAll = lchoose(bg_size,size);

  for( i1=0 ; i1<=max ; i1++ ) {
	if( (i1>bg1) || (i1>size) )
	  break;
	for( i2=0 ; i2<=(max-i1)/2 ; i2++ ) {
	  if( (i2>bg2) || ((i1+i2)>size) )
		break;
	  for( i3=0 ; i3<=(max-i1-2*i2)/3 ; i3++ ) {
		if( (i3>bg3) || ((i1+i2+i3)>size) )
		  break;
		if( (size-i1-i2-i3)>bg0 )
		  continue;
		if( (i1+2*i2+3*i3)>=hits ) {
		  prob += exp(lchoose(bg1,i1)+lchoose(bg2,i2)+lchoose(bg3,i3)+lchoose(bg0,(size-i1-i2-i3))-logAll);
		}
	  }
	}
  }
  if( prob>1 )
	prob = 1.;
  return prob;
}




double calc_hgs(int grp_hit,int grp_tot,int bck_hit,int bck_tot){
	//stat[0] - hits in group; stat[1] - size of group; 
	//stat[2] - size of bg; stat[3] - hits in bg
	int stat[6];
	stat[0]=grp_hit;
	stat[1]=grp_tot;
	stat[2]=bck_tot;
	stat[3]=bck_hit;
	stat[4]=0;stat[5]=0;
	//printf("\nstat= hit in group=%d,  size of group=%d, size bg=%d, hits bg=%d, hg=%f",stat[0],stat[1],stat[2],stat[3],1000*enrich_prob(stat));
	return enrich_prob(stat);
}




void my_exit( char *str )
{
  printf("%s\n", str);
  exit(-1);
}

void bzero(void *p, int n)
{	memset(p, 0, n);}
void foo(int *x){
	for(int i=0;i<5;i++)x[i]=i;
}


void sort( int arr[], int size )
{ 
   int indexOfMin; 
   int pass; 
   int j,tmp; 
   for ( pass = 0; pass < size - 1; pass++ ) 
   { 
        indexOfMin = pass; 
			for ( j = pass + 1; j < size; j++ ) 
            if ( arr[j] < arr[indexOfMin] )indexOfMin = j; 
        SWAP( arr[pass], arr[indexOfMin],tmp ); 
   } 

}



void sortD( double arr[], int size ) 
{ 
   int indexOfMin; 
   int pass; 
   int j;
   double tmp; 
   for ( pass = 0; pass < size - 1; pass++ ) 
   { 
        indexOfMin = pass; 
			for ( j = pass + 1; j < size; j++ ) 
				if ( arr[j] < arr[indexOfMin] )indexOfMin = j; 
        SWAP( arr[pass], arr[indexOfMin],tmp ); 
   } 
   for(int i=0;i<size;i++){if(i>0&& arr[i-1]>arr[i])my_exit("1.sort error1\n");}
}

void  insortD (double *array,int len)
{
	register int i, j;double temp;
	for (i = 1; i < len; i++) {
		j = i;
		temp = array[j];
		while (j > 0 && array[j-1]>temp) {
			array[j] = array[j-1];
			j--;
		}
		array[j] = temp;
	}
	for(int i=0;i<len;i++){if(i>0&& array[i-1]>array[i])my_exit("2.sort error2\n");}
}

double my_drand48()
{
	double r = rand() % 0xffff;
	return r/(double)0xffff;
}





void merge(double numbers[], double temp[], int labels[],int tmplabels[],int left, int mid, int right)
{
  int i, left_end, num_elements, tmp_pos;
  left_end = mid - 1;
  tmp_pos = left;
  num_elements = right - left + 1;

  while ((left <= left_end) && (mid <= right))
  {
    if (numbers[left] <= numbers[mid])
    {
      temp[tmp_pos] = numbers[left];
      tmplabels[tmp_pos] = labels[left];
      tmp_pos = tmp_pos + 1;
      left = left +1;
    }
    else
    {
      temp[tmp_pos] = numbers[mid];
      tmplabels[tmp_pos] = labels[mid];
      tmp_pos = tmp_pos + 1;
      mid = mid + 1;
    }
  }

  while (left <= left_end)
  {
    temp[tmp_pos] = numbers[left];
    tmplabels[tmp_pos] = labels[left];
    left = left + 1;
    tmp_pos = tmp_pos + 1;
  }
  while (mid <= right)
  {
    temp[tmp_pos] = numbers[mid];
    tmplabels[tmp_pos] = labels[mid];
    mid = mid + 1;
    tmp_pos = tmp_pos + 1;
  }
  
  for (i=0; i < num_elements; i++)
  {
    numbers[right] = temp[right];
    labels[right] = tmplabels[right];
    //if(right<0){printf("Error, r=%d l=%d m=%d num_e=%d\n",r,l,m,num_elements);fflush(stdout);}
    right = right - 1;
  }
}






void m_sort(double numbers[], double temp[], int labels[], int tmplabels[],int left, int right)
{
  int mid;

  if (right > left)
  {
    mid = (right + left) / 2;
    m_sort(numbers, temp,labels, tmplabels,left, mid);
    m_sort(numbers, temp,labels, tmplabels,mid+1, right);

    merge(numbers, temp,labels,tmplabels, left, mid+1, right);
  }
}

void mergeSort(double numbers[], int array_size)
{
  int *labels=new int[array_size];
  for(int i=0;i<array_size;i++)labels[i]=i;
  int *tmplabels=new int [array_size];double *temp=new double[array_size];
  m_sort(numbers, temp,labels,tmplabels, 0, array_size - 1);
  delete[]labels;delete[]tmplabels;delete[] temp;
}

void mergeSort(double numbers[], int labels[],int array_size)
{
  for(int i=0;i<array_size;i++)labels[i]=i;
  int *tmplabels=new int[array_size];
  double *temp=new double[array_size];
  m_sort(numbers, temp,labels,tmplabels, 0, array_size - 1);
  delete[]tmplabels;delete[]temp;
}

void mergeSort(double numbers[], int labels[],int array_size,bool useLabels)
{
  if(!useLabels)for(int i=0;i<array_size;i++)labels[i]=i;
  int *tmplabels=new int[array_size];
  double *temp=new double[array_size];
  m_sort(numbers, temp,labels,tmplabels, 0, array_size - 1);
  delete[]tmplabels;delete[]temp;
}

/*void mergeSortInd(double numbers[], int labels[],int array_size)
{
  int tmplabels[array_size];double temp[array_size];
  m_sort(numbers, temp,labels,tmplabels, 0, array_size - 1);
}*/
double FDR(double * arr1,int len,int offset,double FDR_TH){
	//double FDR_TH=0.05;
	double *arr=(double*)malloc(sizeof(double)*len+1);
	int lables[len];
	for(int i=0;i<len;i++)arr[i]=arr1[i];
	mergeSort(arr,lables,len);
	for(int i=0;i<len;i++){if(i>0&& arr[i-1]>arr[i])
		{for(int i=0;i<len;i++){printf("%g ",arr[i]);fflush(stdout);}my_exit("3.sort error\n");}}
	double ans=-1.;
	for(int k1=(len);k1>0;k1--){
		if(arr[k1-1]*(double)((double)(len+offset)/(double)k1)<=FDR_TH)
		{ans=arr[k1-1];break;}
	}
	free(arr);
	return ans;
}












int strfind(const char *as1, const char *as2)
{
     	const char	*s1, *s2;
     	char c;
     	int	offset;
     
     	s1 = as1;
     	s2 = as2;
     	c = *s2;
     
     	while (*s1)
     		if (*s1++ == c) {
     			offset = s1 - as1 - 1;
     			s2++;
     			while ((c = *s2++) == *s1++ && c)
     				;
     			if (c == 0)
     				return (offset);
     			s1 = offset + as1 + 1;
     			s2 = as2;
     			c = *s2;
     		}
     	return (-1);
}
int find(int e,int arr[],int arrlen){
	for(int i=0;i<arrlen;i++)
		if(arr[i]==e)return i;
	return -1;
}
bool is_member(int e,int arr[],int arrlen){
	for(int i=0;i<arrlen;i++)
		if(arr[i]==e)return true;
	return false;
}
bool is_member(void* e,void* arr[],int arrlen){
	for(int i=0;i<arrlen;i++)
		if(arr[i]==e)return true;
	return false;
}
int intersect(void* arr1[],void* arr2[],int len){
	//Assume elements are distinct
	int cntr=0;
	for(int i=0;i<len;i++)if(is_member(arr1[i],arr2,len))cntr++;
	return cntr;
}

void mult(double **mat,double *vec,int n,int m,double *ans){
	//multiplies nxm matrix in a mx1 vector
	for(int i=0;i<n;i++){
		ans[i]=0;
		for(int j=0;j<m;j++)ans[i]+=mat[i][j]*vec[j];
	}
}

void mult(double **mat1,double **mat2,double **res,int n){
	//multiplies two nxn matrices, first matrix is sparse
	for(int i=0;i<n;i++){
		for(int j=0;j<n;j++){
			res[i][j]=0;
			for(int k=0;k<nodeList[i]->deg;k++)
				res[i][j]+=mat1[i][k]*mat2[matrixind[i][k]][j];
		}
	}
}
double norm1(double *vec,int n){
	double res=0;
	for(int i=0;i<n;i++)res+=fabs(vec[i]);
	return res;
}

double norm2(double *vec,int n){
	double res=0;
	for(int i=0;i<n;i++)res+=pow(vec[i],2);
	return sqrt(res);
}

void vec_minus(double *vec1,double *vec2,double *resvec,int n){
	for(int i=0;i<n;i++)resvec[i]=vec1[i]-vec2[i]; 
}
void vec_plus(double *vec1,double *vec2,double *resvec,int n){
	for(int i=0;i<n;i++)resvec[i]=vec1[i]+vec2[i]; 
}
void mul_scalar_vec(double *vec1,double c,double *resvec,int n){
	for(int i=0;i<n;i++)resvec[i]=vec1[i]*c; 
}
void vec_minus(double *vec1,double *vec2,int n){
	for(int i=0;i<n;i++)vec1[i]-=vec2[i]; 
}
void vec_plus(double *vec1,double *vec2,int n){
	for(int i=0;i<n;i++)vec1[i]+=vec2[i]; 
}
void vec_plus(double *vec1,double num,int n){
	for(int i=0;i<n;i++)vec1[i]+=num; 
}
void mat_plus(double **mat,double num,int n,int m){
	for(int i=0;i<n;i++)
		for(int j=0;j<m;j++)
			mat[i][j]+=num; 
}
void mat_plus(double **mat,double **mat2,int n){
	for(int i=0;i<n;i++)
		for(int j=0;j<n;j++)
			mat[i][j]+=mat2[i][j]; 
}
void mul_scalar_vec(double *vec1,double c,int n){
	for(int i=0;i<n;i++)vec1[i]*=c; 
}
void mul_scalar_mat(double **mat,double c,int n,int m){
	for(int i=0;i<n;i++)
		for(int j=0;j<m;j++)mat[i][j]*=c; 
}
void add_scalar_mat(double **mat,double c,int n,int m){
	for(int i=0;i<n;i++)
		for(int j=0;j<m;j++)mat[i][j]+=c; 
}
void transpose(double**mat,int dim){
	double tmp;
	for(int i=0;i<dim;i++){
		for(int j=i;j<dim;j++){
			SWAP(mat[i][j],mat[j][i],tmp);	
		}
	}
}
double mean(double *vec,int n){
	double ans=0;
	for(int i=0;i<n;i++)ans+=vec[i];
	return ans/n;
}
double mean(int *vec,int n){
	double ans=0;
	for(int i=0;i<n;i++)ans+=vec[i];
	return ans/n;
}


double median(double *vec,int n){
	sortD(vec,n);
	return vec[(int)n/2];	
}
int median(int *vec,int n){
	sort(vec,n);
	return vec[(int)n/2];	
}
double stdv(double *vec,int n){
	double ans=0;
	double mn=mean(vec,n);
	for(int i=0;i<n;i++)ans+=pow(vec[i]-mn,2.);
	return sqrt(ans/(n));
}
double get_abs(double x){
	if(x<0)return -x;
	return x;	
}
double cov(double *vec1,double *vec2,int n){
	double ans=0;
	double mn1=mean(vec1,n),mn2=mean(vec2,n);
	for(int i=0;i<n;i++)ans+=(vec1[i]-mn1)*(vec2[i]-mn2);
	return ans/n;
}
bool is_eig_vec(double **mat,double *vec,int n,int m){
	double tmpvec[m],tmpvec2[m];int cntr=0;
	mult(mat,vec,n,m,tmpvec);
	for(int i=0;i<m;i++)
		if(fabs(vec[i])>1e-8)tmpvec2[cntr++]=tmpvec[i]/vec[i];
	double st=stdv(tmpvec2,cntr);
	printf("is_eig_vec: Std= %g, cand eigenValue=%g (nnz=%d)\n",st,mean(tmpvec2,cntr),cntr);
	return st<EPSILON;
}

bool is_symmetric(double **mat,int n){
	for(int j=0;j<n;j++){
		for(int i=0;i<n;i++){
			if(mat[i][j]!=mat[j][i])return false;
		}
	}return true;
}

void column_normalize(double **mat,int n,int m){
	//Column normalization nxm matrix
	double sum[m];
	for(int j=0;j<m;j++){
		sum[j]=0;
		for(int i=0;i<n;i++){
			if(mat[i][j]>0){sum[j]+=mat[i][j];}
		}
	}
	int nz=0;
	for(int j=0;j<m;j++){
		if(sum[j]==0){nz++;continue;}
		for(int i=0;i<n;i++){
			mat[i][j]/=sum[j];
		}
	}
	if(nz>0)printf("warning: found %d all-zero columns\n",nz);
}



void normalize_vec_nrm1(double *vec,int n){
	double x=norm1(vec,n);
	if(x<EPSILON){printf("Warning  - trying to normalize zero vector\n");return;}
	for(int i=0;i<n;i++)vec[i]/=x;
}
void normalize_vec_nrm2(double *vec,int n){
	double x=norm2(vec,n);
	if(x<EPSILON){printf("Warning  - trying to normalize zero vector\n");return;}
	for(int i=0;i<n;i++)vec[i]/=x;
}

void get_min_max(double *vec,int n,double *mn,double *mx){
	*mn=INF;*mx=-INF;
	for(int i=0;i<n;i++){
		if(*mn>vec[i])*mn=vec[i];
		if(*mx<vec[i])*mx=vec[i];
	}
}
double get_max(double *vec,int n){
	double mx=-INF;
	for(int i=0;i<n;i++){
		if(mx<vec[i])mx=vec[i];
	}
	return mx;
}

int get_max(int *vec,int n){
	int mx=-9999999;
	for(int i=0;i<n;i++){
		if(mx<vec[i])mx=vec[i];
	}
	return mx;
}

int get_max(int *vec,int n,int *ind){
	int mx=-9999999;int mxind=-1;
	for(int i=0;i<n;i++){
		if(mx<vec[i]){mxind=i;mx=vec[i];}
	}
	*ind=mxind;
	return mx;
}

double get_max(double *vec,int n,int *ind){
	double mx=-9999999;int mxind=-1;
	for(int i=0;i<n;i++){
		if(mx<vec[i]){mxind=i;mx=vec[i];}
	}
	*ind=mxind;
	return mx;
}

double get_min(double *vec,int n){
	double mn=INF;
	for(int i=0;i<n;i++){
		if(mn>vec[i])mn=vec[i];
	}
	return mn;
}
double get_min(double *vec,int n,int *ind){
	double mn=INF;int mnind=-1;
	for(int i=0;i<n;i++){
		if(mn>vec[i]){mnind=i;mn=vec[i];}
	}
	*ind=mnind;
	return mn;
}
double corrcoef(double *vec1,double *vec2,int n){
	return cov(vec1,vec2,n)/(stdv(vec1,n)*stdv(vec2,n));
}
void  matrix_to_lower_triangle(double **matrix,double*  lower_triangle,int matrix_size)
{
  int i_row;
  int i_col;
  int array_index;

  array_index = 0;
  for (i_row = 0; i_row < matrix_size; i_row++) {
    for (i_col = 0; i_col <= i_row; i_col++) {
      lower_triangle[array_index] = matrix[i_row][i_col];
      array_index++;
    }
  }
  if(2 * array_index != matrix_size * (matrix_size + 1))
  	my_exit("matrix_to_lower_triangle: integrity\n");
}
int ismember(int e,int arr[],int len){
	for(int i=0;i<len;i++)
		if(arr[i]==e)return i;
	return -1;	
}

double roc(double *vec1,bool *labels1,int N){
	//the smaller - the better chance to be labeled as 1
	int num_true=0,num_false=0;
	int *labels=new int[N];
	double *vec=new double[N];
	for(int i=0;i<N;i++){
		if(labels1[i])num_true++;
		else num_false++;
		labels[i]=(int)labels1[i];
		vec[i]=vec1[i];
	}
	if(num_true==0){
		if(num_false>0)return 0;
		else return NAN;	
	}
	if(num_false==0)return 1; //num_true!=0
			
	double *uarr=(double*)malloc(sizeof(double)*N);
	mergeSort(vec,labels,N,true);
	int k=0;uarr[0]=vec[0];
	for(int i=1;i<N;i++){
		if(uarr[k]<vec[i])uarr[++k]=vec[i];
		else if(uarr[k]>vec[i]){
			for(int ii=1;ii<N;ii++)printf("%g ",vec[ii]);
			my_exit("\n4.sort error\n");
		}
	}
	int N1=k+1;int RES=1;int strt=0;int i;
	double fp=0,tp=0,prev_tp,prev_fp,roc1=0,th;
	for(int k=0;k<N1+RES-1;k+=RES){
		if(k>=N1)th=uarr[N1-1];else th=uarr[k];
		prev_fp=fp;prev_tp=tp;
		for(i=strt;i<N && vec[i]<=th;i++){
			if(labels[i]==1)tp++;else fp++;
		}strt=i;
		//printf("th=%g, tp=%d, fp=%d, nt=%d,nneg=%d\n",th,(int)tp,(int)fp,num_true,num_false);
		roc1+=((fp-prev_fp)/(double)num_true) * ((tp+prev_tp)/(double)num_false) *0.5;
		//printf(", roc=%g\n",roc);
	}
	delete[]labels;delete[]vec;
	return roc1;
}


double roc(double *vec1,bool *labels1,int N,int num_false,int num_true){
	//the smaller - the better chance to be labeled as 1
	int *labels=new int[N];
	double *vec=new double[N];
	for(int i=0;i<N;i++){vec[i]=vec1[i];labels[i]=(int)labels1[i];}
	if(num_true==0){
		if(num_false>0){printf("Warning ROC with no positives\n");return 0;}
		printf("Warning ROC with no data\n");
		return NAN;	
	}
	if(num_false==0){
		printf("Warning ROC with no negatives\n");
		return 1; //num_true!=0
	}
			
	double *uarr=(double*)malloc(sizeof(double)*N);
	mergeSort(vec,labels,N,true);
	int k=0;uarr[0]=vec[0];
	for(int i=1;i<N;i++){
		if(uarr[k]<vec[i])uarr[++k]=vec[i];
		else if(uarr[k]>vec[i]){
			for(int ii=1;ii<N;ii++)printf("%g ",vec[ii]);
			my_exit("\n4.sort error\n");
		}
	}
	int N1=k+1;int RES=1;int strt=0;int i;
	double fp=0,tp=0,prev_tp,prev_fp,roc1=0,th;
	for(int k=0;k<N1+RES-1;k+=RES){
		if(k>=N1)th=uarr[N1-1];else th=uarr[k];
		prev_fp=fp;prev_tp=tp;
		for(i=strt;i<N && vec[i]<=th;i++){
			if(labels[i]==1)tp++;else fp++;
		}strt=i;
		//printf("th=%g, tp=%d, fp=%d, nt=%d,nneg=%d\n",th,(int)tp,(int)fp,num_true,num_false);
		roc1+=((fp-prev_fp)/(double)num_true) * ((tp+prev_tp)/(double)num_false) *0.5;
		//printf(", roc=%g\n",roc);
	}
	delete[]labels;delete[]vec;
	return roc1;
}

double detailed_roc(double *vec,bool *labels1,double **tp_arr1,double **fp_arr1,
	double **prec_arr1,int *len,int N,int num_false,int num_true){
	//the smaller - the better chance to be labeled as 1
	int *labels=new int[N];
	for(int i=0;i<N;i++)labels[i]=(int)labels1[i];
	
	if(num_true==0){
		if(num_false>0){printf("Warning ROC with no positives\n");return 0;}
		printf("Warning ROC with no data\n");
		return NAN;	
	}
	if(num_false==0){
		printf("Warning ROC with no negatives\n");
		(*tp_arr1)=new double [1];
		(*fp_arr1)=new double [1];
		(*prec_arr1)=new double [1];
		(*tp_arr1)[0]=1;
		(*fp_arr1)[0]=1;
		(*prec_arr1)[0]=1; //T/T+F
		return 1; //num_true!=0
	}
			
	double *uarr=(double*)malloc(sizeof(double)*N);
	mergeSort(vec,labels,N,true);
	int k=0;uarr[0]=vec[0];
	for(int i=1;i<N;i++){
		if(uarr[k]<vec[i])uarr[++k]=vec[i];
		else if(uarr[k]>vec[i]){
			for(int ii=1;ii<N;ii++)printf("%g ",vec[ii]);
			my_exit("\n4.sort error\n");
		}
	}
	int N1=k+1;int RES=1;int strt=0;int i;int cntr=0;
	double fp=0,tp=0,prev_tp,prev_fp,roc1=0,th;
	double *tp_arr=new double [(int)N1/RES + 10];
	double *fp_arr=new double [(int)N1/RES + 10];
	double *prec_arr=new double [(int)N1/RES + 10];
	for(int k=0;k<N1+RES-1;k+=RES){
		if(k>=N1)th=uarr[N1-1];else th=uarr[k];
		prev_fp=fp;prev_tp=tp;
		for(i=strt;i<N && vec[i]<=th;i++){
			if(labels[i]==1)tp++;else fp++;
		}strt=i;
		//printf("th=%g, tp=%d, fp=%d, nt=%d,nneg=%d\n",th,(int)tp,(int)fp,num_true,num_false);
		roc1+=((fp-prev_fp)/(double)num_true) * ((tp+prev_tp)/(double)num_false) *0.5;
		prec_arr[cntr]=tp/(tp+fp);
		tp_arr[cntr]=tp/(double)num_true;
		fp_arr[cntr++]=fp/(double)num_false;
		//printf(", roc=%g\n",roc);
	}
	(*len)=cntr;
	(*tp_arr1)=tp_arr;(*fp_arr1)=fp_arr;(*prec_arr1)=prec_arr;
	return roc1;
}

double *hist(double *arr,int len,int n_bin){
	double *ans=new double[n_bin];
	double step=(arr[0]-arr[len-1])/n_bin;
	int i=0;
	for(int cntr=1;cntr<=n_bin;cntr++){
		ans[cntr-1]=0;
		while( i<len && (arr[0]+cntr*step)>=arr[i]){i++;ans[cntr-1]++;}
	}
	return ans;	
}

double *hist(double *arr,int len,double *ths, int n_bin){
	double *ans=new double[n_bin];
	int i=0;
	for(int cntr=0;cntr<n_bin;cntr++){
		ans[cntr]=0;
		while( i<len && ths[cntr]>=arr[i]){i++;ans[cntr]++;}
		ans[cntr]/=len;
	}
	return ans;	
}

void flatten ( double *vec, int len ){ 
  /* flatten a vector -- replace entries with mean */
  int i;
  double m = 0.0;
  for ( i = 0 ; i < len ; i++ ) {
   m += vec[i];
  } 
  m = m/len;
  for ( i = 0 ; i < len ; i++ ) {
   vec[i] = m;
  } 
}   

/* for decreasing monotone smooths */
/* the forward-backward pav algorithm */
void pav ( double *result, int length ){ 
  int d, i, j;
  for ( i = 0 ; i < length-1 ; i++ ) {
   if ( result[i] < result[i+1] ) {
     result[i] = (result[i]+result[i+1])/2.0;
     result[i+1] = result[i];
     for ( j = i-1 ; j >= 0 ; j-- ) {
       if ( result[j] < result[i+1] ) {
          d = (i+1)-j+1;
          flatten ( &result[j] , d );
       } else {
          break;
       }  
} }} } 

int average_curve(double **arrX,double **arrY,int *len_arr,int arr_size,
	double **resX1,double **resY1,int resolution){
	resolution++;
	double *resX=new double[resolution+10];
	double *resY=new double[resolution+10];
	double left=0;
	int iter_arr[arr_size];
	double val_arr[arr_size];
	for(int k=0;k<arr_size;k++){iter_arr[k]=0;val_arr[k]=0;}
	int i=0;
	for (double right=1/(double)resolution;right<=1;right+=1/(double)resolution){
		bool updated=false;
		for(int k=0;k<arr_size;k++){
			if(iter_arr[k]==len_arr[k]||(arrX[k][iter_arr[k]]>right&&iter_arr[k]>0)){
				val_arr[k]=arrY[k][iter_arr[k]-1];//stay with last value on previous bin
				continue;
			}
			double val=arrY[k][iter_arr[k]];
			int tmpcntr=1;iter_arr[k]++;
			while(iter_arr[k]<len_arr[k] && arrX[k][iter_arr[k]]<right){
				val+=arrY[k][iter_arr[k]];
				iter_arr[k]++;tmpcntr++;
			}
			updated|=tmpcntr>1;
			val_arr[k]=val/(double)tmpcntr;
		}
		if(updated){
			resY[i]=mean(val_arr,arr_size);
			resX[i++]=(left+right)/2;
		}
		left=right;	
	}
	resY[i]=1;resX[i++]=1;
	*resX1=resX;*resY1=resY;
	return i;
}


/*
void pool_adj_viol(double *arr,int len){
	double mn;int viol=1,iter=0;
	for(;iter<500000&&viol!=0;iter++){
		if(iter%25000==0){printf(".");fflush(stdout);}
		viol=0;
		for(int i=0;i<len-1;i++){
			if(arr[i]-arr[i+1]<-1e-20){
				mn=(arr[i]+arr[i+1])/2;
				arr[i]=mn;arr[i+1]=mn;
				viol++;
			}
		}
	}
	printf("\npool_adj_viol %d itertion, violation %d\n",iter,viol);
}

void pool_adj_viol(double *arr,int len){
	for(int i=0;i<len;i++){
		printf(".");fflush(stdout);
		pool_adj_viol_rec(arr,i,len);
	}
	printf("\n\n");fflush(stdout);
}
void pool_adj_viol_rec(double *arr,int loc,int len){
	if(loc==len-1 || loc==-1)return;
	while(arr[loc]-arr[loc+1]<-1e-20)
	//while(arr[loc]<arr[loc+1]){
		double mn=(arr[loc]+arr[loc+1])/2;
		arr[loc]=mn;arr[loc+1]=mn;
		pool_adj_viol_rec(arr,loc-1,len);	
	}
}
*/

#endif /*UTIL_CPP_*/
